"""
-----------------------
Author: Sukumar Kutagulla (Read-only)
Designation: Test Automation Architect (Read-only)
Copyright (c) 2025. All Rights Reserved
-----------------------
"""

__version__ = "0.1.2"

from .cli.main import main