"""
-----------------------
Author: Sukumar Kutagulla (Read-only)
Designation: Test Automation Architect (Read-only)
Copyright (c) 2025. All Rights Reserved.
-----------------------

"""

__author__ = "Sukumar Kutagulla"
__author_title__ = "Test Automation Architect"
__copyright__ = f"Copyright (c) 2025, {__author__}. All Rights Reserved."

from .custom_actions import TestController

__all__ = ['TestController']